using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using EfCoreStoreProc.Controllers;
using EfCoreStoreProc.Data;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Xunit;

namespace EfCoreStoredProc.Tests
{
    public class ProductsControllerTests
    {
        [Fact]
        public async Task Get_Results()
        {
            //Arrange 
            var products = new SpAsyncEnumerableQueryable<Product>(new Product()
            {
                ProductId = Guid.NewGuid(),
                ProductName = "Some name",
                Size = 1,
                Value = 2
            }, new Product()
            {
                ProductId = Guid.NewGuid(),
                ProductName = "Some other name",
                Size = 3,
                Value = 4
            });

            var productContextOptions = new DbContextOptionsBuilder<ProductContext>()
                .UseInMemoryDatabase(databaseName: "Get results")
                .Options;

            ProductContext productContext = new ProductContext(productContextOptions);

            productContext.Products = productContext.Products.MockFromSql(products);

            ProductsController controller = new ProductsController(productContext);

            //Act
            IActionResult actionResult = await controller.Get();

            //Assert
            OkObjectResult okObjectResult = actionResult as OkObjectResult;
            Assert.NotNull(okObjectResult);
            List<Product> productResponse = okObjectResult.Value as List<Product>;

            Assert.Equal(2, productResponse.Count);
        }

        [Fact]
        public async Task Get_NoResults()
        {
            //Arrange 
            var products = new SpAsyncEnumerableQueryable<Product>();

            var productContextOptions = new DbContextOptionsBuilder<ProductContext>()
                .UseInMemoryDatabase(databaseName: "Get results")
                .Options;

            ProductContext productContext = new ProductContext(productContextOptions);

            productContext.Products = productContext.Products.MockFromSql(products);

            ProductsController controller = new ProductsController(productContext);

            //Act
            IActionResult actionResult = await controller.Get();

            //Assert
            NoContentResult noContentResult = actionResult as NoContentResult;
            Assert.NotNull(noContentResult);
        }
    }
}
